/*
 *
 * Copyright (c) 2025. Jeffery Hsu - Achievement Report Generator
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

import { IAssessment, IAssessmentMethod } from '@renderer/types'

export class AssessmentData implements IAssessment {
  private readonly _method: IAssessmentMethod[]
  private _stage: string
  private _weight: number

  constructor(stage: string, weight: number | string, method?: IAssessmentMethod[]) {
    this._method = method ?? []
    this._stage = stage
    this._weight = Number(weight)
  }

  get method(): IAssessmentMethod[] {
    return this._method
  }

  get stage(): string {
    return this._stage
  }

  set stage(value: string) {
    this._stage = value
  }

  get weight(): number {
    return this._weight
  }

  set weight(value: number | string) {
    this._weight = Number(value)
  }

  public addMethod(method: IAssessmentMethod) {
    this._method.push({ ...method, weight: Number(method.weight) })
  }

  public modifyMethod(method: IAssessmentMethod, idx: number) {
    if (idx < 0 || idx >= this._method.length) {
      return
    }
    this._method[idx] = { ...method, weight: Number(method.weight) }
  }

  public removeMethod(idx: number) {
    this.method.splice(idx, 1)
  }

  public static factory() {
    return new AssessmentData('', 0, [])
  }
}

export class Assessment {
  private _daily: AssessmentData
  private _progress: AssessmentData
  private _final: AssessmentData

  constructor(daily: AssessmentData, progress: AssessmentData, final: AssessmentData) {
    this._daily = daily
    this._progress = progress
    this._final = final
  }

  public static factory() {
    return new Assessment(
      AssessmentData.factory(),
      AssessmentData.factory(),
      AssessmentData.factory()
    )
  }

  get daily(): AssessmentData {
    return this._daily
  }

  set daily(value: AssessmentData) {
    this._daily = value
  }

  get progress(): AssessmentData {
    return this._progress
  }

  set progress(value: AssessmentData) {
    this._progress = value
  }

  get final(): AssessmentData {
    return this._final
  }

  set final(value: AssessmentData) {
    this._final = value
  }
}
